import static java.lang.Math.*;


public class vectorFunc{
    /*
    */
    public static vector add(vector a, vector b)
    {
        return new vector( a.x+b.x, a.y+b.y);
    }

    public static vector sub(vector a, vector b)
    {
        return new vector( a.x-b.x, a.y-b.y);
    }

    public static vector mul(vector a, vector b)
    {
        return new vector( a.x*b.x, a.y*b.y);
    }
    
    public static vector div(vector a, vector b)
    {
        return new vector( a.x/b.x, a.y/b.y);
    }
    
    /*
    */
    
    public static vector scale(vector a, float scalar)
    {
        return new vector( a.x*scalar, a.y*scalar);
    }
    
    public static vector scale(vector a, double scalar)
    {
        return new vector( a.x*scalar, a.y*scalar);
    }
    
    public static float dot(vector a, vector b)
    {
        return (a.x*b.x) + (a.y*b.y);
    }
    
    public static float cross(vector a, vector b)
    {
        return (a.x*b.y) - (a.y*b.x);
    }
    
    /*
    */
    
    public static float magSQ(vector a)
    {
        return (float)(pow(a.x,2.0) + pow(a.y,2.0));
    }
    
    public static float mag(vector a)
    {
        return (float)sqrt(magSQ(a));
    }
    
    public static vector normalize(vector a)
    {
        float scalar = mag(a);
        return new vector( a.x/scalar, a.y/scalar);
    }
    
    public static vector normalizeSQ(vector a)
    {
        float scalarSQ = magSQ(a);
        return new vector( (float)pow(a.x,2.0)/scalarSQ, (float)pow(a.y,2.0)/scalarSQ);
    }
    
    /*
    */
    
    public static vector rotate(vector a, vector r)
    {
        return new vector(a.x * r.y - a.y * r.x, a.x * r.x + a.y * r.y);
    }
    
    public static vector fromRadian(double radian)
    {
        return new vector( sin(radian), cos(radian));
    }
    
    public static float toRadian(vector a)
    {
        return (float)atan2(a.x,a.y);
    }
    
    /*
    */
    
    public static float closestU(vector a, vector p1, vector p2)
    {
        vector delta = sub(p2,p1);
        return dot(sub(a,p1),delta)/magSQ(delta);
    }
    
    public static vector closest(vector a, vector p1, vector p2, float U)
    {
        return add(p1, scale(sub(p2,p1),U));
    }
    
    public static vector closest(vector a, vector p1, vector p2)
    {
        vector delta = sub(p2,p1);
        float u = dot(sub(a,p1), delta)/ magSQ(delta);
        return add(p1, scale(delta,u));
    }
    
    /*
    */
    
    public static float parallelZero(vector p1, vector p2, vector p3, vector p4)
    {
        return cross(sub(p2,p1), sub(p4,p3));
    }
    
    public static float intersectUa(vector p1, vector p2, vector p3, vector p4, float pZero)
    {
        vector delta = sub(p4,p3);
        vector deltaWTF = sub(p1,p3);
        return cross(delta,deltaWTF)/pZero;
    }
    
    public static float intersectUb(vector p1, vector p2, vector p3, vector p4, float pZero)
    {
        vector delta = sub(p2,p1);
        vector deltaWTF = sub(p1,p3);
        return cross(delta,deltaWTF)/pZero;
    }
    
    public static vector intersect(vector p1, vector p2, vector p3, vector p4)
    {
        float pZero = parallelZero(p1,p2,p3,p4);
        float Ua = intersectUa(p1,p2,p3,p4,pZero);
        //float Ub = intersectUb(p1,p2,p3,p4,pZero);
        return add(p1,scale(sub(p2,p1),Ua));
    }
    
    public static vector intersect(vector p1, vector p2, float Ua)
    {
        return add(p1,scale(sub(p2,p1),Ua));
    }
    
};
